Option Strict Off
Option Explicit On
Friend Class serialComm
	Inherits System.Windows.Forms.Form
	Private Declare Function GetTickCount Lib "kernel32" () As Integer
	
	Dim responseText As String
	
	Private Sub clearButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles clearButton.Click
		responseBox.Text = ""
	End Sub
	
	Private Sub closePort_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles closePort.Click
        If SerialPort1.IsOpen Then
            SerialPort1.Close()
            closePort.Enabled = False
            openPort.Enabled = True
            enableChoice((True))
        End If
	End Sub
	
	Private Sub enableChoice(ByRef state As Boolean)
		portChoice.Enabled = state
		baudChoice.Enabled = state
		dataBitChoice.Enabled = state
		stopBitChoice.Enabled = state
		parityChoice.Enabled = state
		timeOutBox.Enabled = state
	End Sub
	
	Private Sub commandString_KeyPress(ByVal eventSender As System.Object, ByVal eventArgs As System.Windows.Forms.KeyPressEventArgs) Handles commandString.KeyPress
		Dim KeyAscii As Short = Asc(eventArgs.KeyChar)
		Dim rstr As String
		If KeyAscii = 13 Then
			KeyAscii = 0 ' get rid of the normal beep sound
            rstr = sendCommand((commandString.Text))
			If rstr = "" Then
				Beep()
				appendlnResponse(("(Warning: No Response from PLC)"))
			Else
				appendlnResponse((rstr))
			End If
		End If
		eventArgs.KeyChar = Chr(KeyAscii)
		If KeyAscii = 0 Then
			eventArgs.Handled = True
		End If
	End Sub
	
	Private Sub exitButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles exitButton.Click
		Call closePort_Click(closePort, New System.EventArgs())
		Hide()
	End Sub
	
    Private Sub serialComm_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        Call serialCommInit()
    End Sub

    Public Sub serialCommInit()
        portChoice.SelectedIndex = 4
        baudChoice.SelectedIndex = 5
        dataBitChoice.SelectedIndex = 1
        stopBitChoice.SelectedIndex = 0
        parityChoice.SelectedIndex = 0
        If SerialPort1.IsOpen Then
            closePort.Enabled = True
            openPort.Enabled = False
        Else
            closePort.Enabled = False
            openPort.Enabled = True
        End If

    End Sub

    Public Function getCommport() As String
        getCommport = portChoice.SelectedItem
    End Function


    Private Sub openPort_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles openPort.Click
        Dim sformat As String

        If SerialPort1.IsOpen Then Exit Sub

         SerialPort1.BaudRate = baudChoice.SelectedItem
        If parityChoice.SelectedIndex = 0 Then
            SerialPort1.Parity = IO.Ports.Parity.None
        ElseIf parityChoice.SelectedIndex = 1 Then
            SerialPort1.Parity = IO.Ports.Parity.Even
        Else
            SerialPort1.Parity = IO.Ports.Parity.Odd
        End If
        If dataBitChoice.SelectedIndex = 1 Then
            SerialPort1.StopBits = IO.Ports.StopBits.One
        Else
            SerialPort1.StopBits = IO.Ports.StopBits.Two
        End If

        SerialPort1.PortName = portChoice.SelectedItem
        SerialPort1.DataBits = dataBitChoice.SelectedItem
        SerialPort1.ReadTimeout = timeOutBox.Text
        On Error GoTo ErrHandler
        SerialPort1.Open()
        appendlnResponse(portChoice.SelectedItem & ": Open at " & baudChoice.SelectedItem)
        SerialPort1.DtrEnable = True

        openPort.Enabled = False
        closePort.Enabled = True
        enableChoice((False))

        Exit Sub
ErrHandler:
        MsgBox(SerialPort1.PortName + " is Unavailable")
    End Sub

    Private Sub appendlnResponse(ByRef s As String)
        responseBox.Text = responseBox.Text & s & Chr(13) & Chr(10)
    End Sub

    ' The following function can be used by user for sending out host link commands
    ' to the PLC. It supports both point-to-point and multi-point version of the
    ' If it does not receive the response string after a specific period defined in
    ' "timeOutBox" it will return with an empty string.

    Public Function openCommPort() As Boolean
        ' If portChoice.SelectedIndex = -1 Then Call serialCommInit()
        Call openPort_Click(openPort, New System.EventArgs())
        If openPort.Enabled = True Then
            openCommPort = False
            openPort.Enabled = True
            closePort.Enabled = False
        Else
            openCommPort = True
            openPort.Enabled = False
            closePort.Enabled = True
        End If
    End Function


    Public Function sendCommand(ByRef cmd As String) As String
        Dim duration As Integer

        If SerialPort1.IsOpen = False Then
            openCommPort()
        End If

        If Mid(cmd, 1, 1) <> "@" And Mid(cmd, 1, 1) <> ":" Then
            SerialPort1.Write(Chr(5))  ' send ctrl-E
            If getCtrlE() = False Then ' should get a Ctrl-E in return
                responseText = ""
                sendCommand = ""
                Exit Function
            End If
        End If
        If Mid(cmd, 1, 1) = ":" Then
            SerialPort1.Write(cmd & Chr(13) & Chr(10))
        Else
            SerialPort1.Write(cmd & Chr(13))
        End If
        On Error GoTo timeOutHandler
        Call getResponse() ' get a response within timeout.
        sendCommand = responseText ' returned received string.
        Exit Function
timeOutHandler:
        responseText = ""
        sendCommand = ""
    End Function

    ' This function append the ID and the FCS to the input string "cmd" and return
    ' the resultant string. cmd should not include the "*" character

    Public Function appendFCS(ByRef ID As Short, ByRef cmd As String) As String
        Dim IDString As String
        Dim FCSString As String
        IDString = Hex(ID)
        If Len(IDString) = 1 Then IDString = "0" & IDString
        cmd = "@" & IDString & cmd
        'UPGRADE_WARNING: Couldn't resolve default property of object computeFCS(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
        FCSString = Hex(computeFCS(cmd))
        If Len(FCSString) = 1 Then FCSString = "0" & FCSString
        cmd = cmd & FCSString & "*"
        appendFCS = cmd ' returned string now contains @ID and FCS+"*"
    End Function

    ' This function compute the FCS for the given input string "cmd" and return the
    ' computed FCS as an integer. This function may be called for verifying whether
    ' the received response string is error free.

    Public Function computeFCS(ByRef cmd As String) As Integer
        Dim result As Short
        Dim length As Short
        Dim I As Short
        length = Len(cmd)
        result = Asc(Mid(cmd, 1, 1))
        For I = 2 To length
            result = result Xor Asc(Mid(cmd, I, 1))
        Next
        computeFCS = result
    End Function

    ' This function wait for a valid Ctrl-E from the PLC and return TRUE. If it does not
    ' receive within the specified timeout then it returns a FALSE

    Private Function getCtrlE() As String
        Dim inbuff As String
        Dim pcount As Integer
        Dim ccount As Integer
        Dim duration As Integer
        On Error GoTo timeOutHandler
        inbuff = SerialPort1.ReadByte()
        If inbuff = "5" Then
            getCtrlE = True
        Else
            getCtrlE = False
        End If
        Exit Function
timeOutHandler:
        getCtrlE = False
    End Function
    ' This procecedure examine the serial input buffer to see if a CR-terminated string
    ' has been received. Once a completed string is received it is copied to the global variable
    ' "responseText" and the buffer will be cleared. If a completed string is not received
    ' within the time-out period then the procedure will return and the "responseText" string
    ' will remain an empty string

    Private Sub getResponse()
        Static responseBuffer As String
        Dim inchar As Byte
        Dim length As Short
        Dim pcount As Integer
        Dim ccount As Integer
        responseBuffer = ""
        Do While True
            On Error GoTo timeoutHandler
            inchar = SerialPort1.ReadByte()
            If inchar = 13 Then
                If Mid(responseBuffer, 1, 1) = ":" Then   ' Modbus command
                    inchar = SerialPort1.ReadByte()
                    If inchar = 10 Then
                        responseText = responseBuffer
                        Exit Sub
                    End If
                Else
                    responseText = responseBuffer
                    Exit Sub
                End If
            End If
            responseBuffer = responseBuffer & Chr(inchar)
        Loop

timeoutHandler:
        responseText = ""
        responseBuffer = ""
    End Sub

    'UPGRADE_WARNING: Event portChoice.TextChanged may fire when form is initialized. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="88B12AE1-6DE0-48A0-86F1-60C0686C026A"'
    'UPGRADE_WARNING: ComboBox event portChoice.Change was upgraded to portChoice.TextChanged which has a new behavior. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="DFCDE711-9694-47D7-9C50-45A99CD8E91E"'
    Private Sub portChoice_TextChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles portChoice.TextChanged
        '  Call FXSvrConfig.displayCommport()
    End Sub
End Class